const St = imports.gi.St;
const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

let corner_types = ['tl', 'tr', 'bl', 'br'];

let corners = {}; // List of corners displayed.
let radius = 16;
let settings;
let signalID;

function init() {}

function enable()
{
    settings = Convenience.getSettings();
    radius = settings.get_int('corner-size');
    settings.connect('changed::corner-size', _setSize);
    signalID = Main.layoutManager.connect('monitors-changed', _reload);
    _createCorners();
}

function disable()
{
    settings.run_dispose();
    Main.layoutManager.disconnect(signalID);
    _destroyCorners();
}

function _destroyCorners()
{
    for (let c in corners)
    { // Destroy the corners.
        corners[c].destroy();
    }
}

function _createCorners()
{
    let monitors = Main.layoutManager.monitors;

    for (let m in monitors)
    { // Multi-monitor support. AFAIK it works but I can't actually test it for like 3 weeks.
        let monitor = monitors[m];

        for (let c in corner_types)
        {
            let corner = corner_types[c];

            corners[monitor.index+corner] = new St.Bin({
                style_class: 'corner'+corner, // Each corner gets a class to tie it to the corner it's in.
                reactive: false,
                can_focus: false, // I think all this says "Don't let me bother anyone" but I can't find any documentation anywhere.
                x_fill: true,
                y_fill: false,
                track_hover: false
            });
        }

        corners[monitor.index+'tl'].x = monitor.x; // Mark each corner with its position.
        corners[monitor.index+'tl'].y = monitor.y;

        corners[monitor.index+'tr'].x = monitor.x+monitor.width-radius;
        corners[monitor.index+'tr'].y = monitor.y;

        corners[monitor.index+'bl'].x = monitor.x;
        corners[monitor.index+'bl'].y = monitor.y+monitor.height-radius;

        corners[monitor.index+'br'].x = monitor.x+monitor.width-radius;
        corners[monitor.index+'br'].y = monitor.y+monitor.height-radius;
    }
    
    for (let c in corners)
    { // Activate the corners.
        Main.uiGroup.add_actor(corners[c]);
        corners[c].set_position(corners[c].x, corners[c].y);
        corners[c].set_style('width: '+radius+'px; height: '+radius+'px; background-size: '+radius+'px;');
    }
}

function _reload()
{ // You know I thought this would be awful but it works pretty well actually.
    _destroyCorners();
    _createCorners();
}

function _setSize()
{
    radius = settings.get_int('corner-size');

    let monitors = Main.layoutManager.monitors;
    for (let m in monitors)
    { // Update positions
        let monitor = monitors[m];

        corners[monitor.index+'tl'].x = monitor.x; // Mark each corner with its position.
        corners[monitor.index+'tl'].y = monitor.y;

        corners[monitor.index+'tr'].x = monitor.x+monitor.width-radius;
        corners[monitor.index+'tr'].y = monitor.y;

        corners[monitor.index+'bl'].x = monitor.x;
        corners[monitor.index+'bl'].y = monitor.y+monitor.height-radius;

        corners[monitor.index+'br'].x = monitor.x+monitor.width-radius;
        corners[monitor.index+'br'].y = monitor.y+monitor.height-radius;
    }

    for (let c in corners)
    { // Update sizes
        corners[c].set_position(corners[c].x, corners[c].y);
        corners[c].set_style('width: '+radius+'px; height: '+radius+'px; background-size: '+radius+'px;');
    }
}
